'''
single_segment_example.py

Author: JP Carrico, JP@SEE.COM
Copyright: Space Exploration Engineering, LLC, 2019
Description:
	An example script that demonstrates how to use STK/Astrogator's
	single segment mode. This file can be run directly if Python 3.x
	is installed.
	
	Set the is_STK_visible flag (line 28, below) to False to run in headless mode.
	
	Requires the Single Segment Example Scenario
	directory to be in the same directory as the .py file, i.e.:
		\single_segment_example.py
		\Single Seg Example Scenario
			\SingleSegmentExampleScenario.sc
			...

'''
# Import Libraries to control STK
import os
import win32com.client as win32
# Set the flag on the following line to indicate whether the STK GUI should launch 
# NB: setting stk_app.Visible = False will hide the GUI, however some graphics 
# calculations will still take place. It's therefore preferable to use an STKX11
# application with NoGraphics set to false, which provides true performance improvements. 
is_STK_visible = True
# Start STK, or get handle if it's already running
if is_STK_visible == True :
    stk_app = win32.dynamic.Dispatch('STK11.Application')
    # Get pointers to the base objects
    root = stk_app.Personality2 # Personality2 is used for the rest of STK (but it can be also used for Astrogator)
    # Flag to make the GUI visible/not visible
    stk_app.Visible = True
else:
    # use python headless instantiation to create a new base object
    stk_app = win32.Dispatch('STKX11.Application')
    stk_app.NoGraphics = 1
    root = win32.Dispatch('AgStkObjects11.AgStkObjectRoot') # This creates the same object that is returned by the Personality2 property 
# Load scenerio. Assumes existence of example scenario directory in the same directory as the notebook.
# Build scenario path
relScenPath = os.path.normpath('Single Seg Example Scenario\SingleSegmentExampleScenario.sc')
fullScenarioPath = os.path.abspath(relScenPath)
print(fullScenarioPath)
# load scenario
root.LoadScenario(fullScenarioPath)
# Get pointer to scenario object
scenario = root.CurrentScenario
# Get pointer to the astrogator satellite
satelliteName = 'GatorSat'
sat = scenario.Children(satelliteName)
# Get the initial state's coordinate system's name using Personality2 
driver = sat.Propagator
initState_seg = driver.MainSequence("Initial State") #IAgVAMCSInitialState Interface
initState_coordSysName = initState_seg.CoordSystemName
print(initState_coordSysName)
# Get pointers to the propagate and maneuver segments
prop_seg = driver.MainSequence("Propagate") #IAgVAMCSPropagate Interface
mnvr_seg = driver.MainSequence("Maneuver")  #IAgVAMCSManeuver Interface
# set initial state values (default units are used)
initState_seg.SetElementType(1) #pass AgEVAElementType Enumeration; 1 = Keplerian
# IAgVAElementKeplerian Interface
initState_seg.Element.SemiMajorAxis = 10000 # uses distance dimension
initState_seg.Element.Eccentricity = 0 # dimensionless
initState_seg.Element.RAAN = 270 # uses angle dimension
initState_seg.Element.Inclination = 30 # uses angle dimension
initState_seg.Element.ArgOfPeriapsis = 40 # uses angle dimension
# Go check out the STK GUI!
mnvr_seg.SetManeuverType(0) # pass AgEVAManeuverType Enumeration; 0 = impulsive delta-v
mnvr = mnvr_seg.Maneuver # IAgVAManeuver Interface
mnvr.SetAttitudeControlType(1) # pass AgEVAAttitudeControl Enumerator; 1 = anti-velocity vector
mnvr.AttitudeControl.DeltaVMagnitude = 0.01 #uses rate dimension, so km/sec
# Start the propagation
driver.BeginRun()
# Run the initial state
initState_seg.Run()
# Run the propagate segment twice (stop cond on periapsis) 
prop_seg.Run()
prop_seg.Run()
# Run the maneuver segment
mnvr_seg.Run()
# Run the propagate segment once (stop cond on periapsis) 
prop_seg.Run()
#End propagation
driver.EndRun()
# Let's check out our new orbit by viewing some results
# on the Propagate segment
new_sma = prop_seg.GetResultValue('Semimajor Axis')
new_ecc = prop_seg.GetResultValue('Eccentricity')
print("New SMA: {} km".format(new_sma))
print("New Eccentricity: {}".format(new_ecc))
#close STK and dereference pointers
root.CloseScenario()
if (is_STK_visible == True): 
    stk_app.Quit()
else:
    stk_app.Terminate()
root = None
stk_app = None